/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/
 
package com.vektorsoft.demux.android.core.gui;

import android.support.v4.app.FragmentActivity;
import com.vektorsoft.demux.core.dlg.DMXDialog;
import com.vektorsoft.demux.core.dlg.DMXDialogView;
import com.vektorsoft.demux.core.dlg.DialogFactory;
import com.vektorsoft.demux.core.mva.DMXView;
import com.vektorsoft.demux.core.mva.DMXViewManager;
import com.vektorsoft.demux.core.resources.DMXResourceManager;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * Factory for creating dialogs on Android.
 *
 * @author Vladimir Djurovic
 */
public class DMXAndroidDialogFactory implements DialogFactory {

    /** Map of previously created dialogs. */
    private final Map<String, DMXDialog> dialogs;
    
    /** View manager. */
    private DMXViewManager viewManager;
    
    /** Parent activity. */
    private FragmentActivity parent;
    
    /** Resource manager. */
    private DMXResourceManager resourceManager;

    /**
     * Creates new instance.
     * 
     * @param viewManager view manager
     * @param parent  parent activity
     */
    public DMXAndroidDialogFactory(DMXViewManager viewManager, FragmentActivity parent) {
        this.viewManager = viewManager;
        this.parent = parent;
        dialogs = Collections.synchronizedMap(new HashMap<String, DMXDialog>());
    }

    /**
     * Set resource manager for this factory.
     * 
     * @param resourceManager resource manager
     */
    public void setResourceManager(DMXResourceManager resourceManager) {
        this.resourceManager = resourceManager;
    }

    
    /**
     * <p>
     * Create custom Android dialog. This dialog will display the view with specified ID. Note that 
     * this view must be instance of {@link DMXDialogView}.
     * </p>
     * <p>
     *  Dialog title is obtained from the view by calling {@code getDialogTitle()} method. Alternatively, caller
     * can override this by specifying alternate title as {@code title} parameter.
     * </p>
     * 
     * @param viewId ID of view to display
     * @param title alternate dialog title
     * @return dialog instance
     */
    public DMXAndroidDialog createDialog(String viewId, String title) {
        DMXAndroidDialog dlg = null;
        DMXView view = viewManager.getView(viewId);
        if (view instanceof DMXDialogView) {
            synchronized (dialogs) {
                if (dialogs.containsKey(viewId)) {
                    dlg = (DMXAndroidDialog)dialogs.get(viewId);
                } else {
                    dlg = new DMXAndroidDialog(parent, (DMXDialogView) view);
                    dialogs.put(viewId, dlg);
                }
            }

        }
        return dlg;
    }
    
    /**
     * Create dialog with specified title, message and predefined options. Parameter {@code options} should 
     * be one of the values defined in {@link DMXDialog}.
     * 
     * @param title dialog title
     * @param message message to display
     * @param options options buttons
     * @return dialog
     */
    public DMXAndroidOptionsDialog createOptionsDialog(String title, String message, int options){
        DMXAndroidOptionsDialog dlg = new DMXAndroidOptionsDialog(parent, resourceManager, title, options, message);
        return dlg;
    }
    
    /**
     * Create dialog with specified title, message and options buttons. Text for options buttons is specified in
     * {@code options} parameter. This is array of maximum 3 elements, representing values for positive, negative
     * and neutral buttons. Element values can either be keys for resource strings or actual text.
     * 
     * @param title dialog title
     * @param message dialog message
     * @param options dialog button text
     * @return dialog
     */
    public DMXAndroidOptionsDialog createOptionsDialog(String title, String message, String[] options){
        return new DMXAndroidOptionsDialog(parent, resourceManager, title, options, message);
    }
    
    /**
     * Create dialog which displays view with specified ID, and with specified options. Parameter {@code options}
     * is the value for options defined in {@link DMXDialog}.
     * 
     * @param viewId view ID
     * @param options dialog options
     * @return  dialog
     */
    public DMXAndroidOptionsDialog createOptionsDialog(String viewId, int options){
        DMXAndroidOptionsDialog dlg = null;
        DMXView view = viewManager.getView(viewId);
        if(view instanceof DMXDialogView){
            dlg = new DMXAndroidOptionsDialog(parent, resourceManager, options, (DMXDialogView)view);
        }
        return dlg;
    }
    
    /**
     * Create dialog which displays view with specified ID, and with specified options. Parameter {@code options}
     * is the array of values for positive, negative and neutral buttons, in that order.
     * 
     * @param viewId view ID
     * @param options options for buttons
     * @return dialog
     */
    public DMXAndroidOptionsDialog createOptionsDialog(String viewId, String[] options){
        DMXAndroidOptionsDialog dlg = null;
        DMXView view = viewManager.getView(viewId);
        if(view instanceof DMXDialogView){
            dlg = new DMXAndroidOptionsDialog(parent, resourceManager, options, (DMXDialogView)view);
        }
        return dlg;
    }
}
