/******************************************************************************
 * 
 * Copyright 2012 - 2014 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.core.app;

import com.vektorsoft.demux.core.extension.DMXExtensionCallback;
import com.vektorsoft.demux.core.extension.DMXExtensionConfig;
import com.vektorsoft.demux.core.mva.DMXAdapter;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.util.tracker.ServiceTrackerCustomizer;

/**
 * <p>
 *  This is an implementation of {@link ServiceTrackerCustomizer} class for {@code DMXAdapter} service. It provides basic 
 * functionality to register extension callbacks from supplied extension configuration. This class is intended to be used in
 * conjunction with {@link DMXAdapterTrackerHandler} for adding custom behavior for service tracker.
 * </p>
 * <p>
 *  The object in {@code handler} field will be used to customize each method, by calling corresponding handler method.
 * </p>
 *
 * @author Vladimir Djurovic
 */
public final class DMXAdapterTrackerCustomizer implements ServiceTrackerCustomizer<DMXAdapter, DMXAdapter> {
    
    /**
     * Bundle context.
     */
    protected BundleContext context;
    
    /**
     * Extension configuration instance.
     */
    protected DMXExtensionConfig extensionConfig;
    
    /**
     * Handler for customizing method behaviors.
     */
    protected DMXAdapterTrackerHandler handler;
    

    /**
     * Creates new instance.
     * 
     * @param ctx bundle context
     * @param config extension configuration
     */
    public DMXAdapterTrackerCustomizer(BundleContext ctx, DMXExtensionConfig config){
        this.extensionConfig = config;
        this.context = ctx;
    }
    
     /**
     * Sets handler instance for this class.
     * 
     * @param handler handler instance
     */
    public void setHandler(DMXAdapterTrackerHandler handler) {
        this.handler = handler;
    }
    

    /**
     * This method will register any extension callbacks specified by extension configuration. Finally, it
     * will invoke {@link DMXAdapterTrackerHandler#onAdapterAdded(com.vektorsoft.demux.core.mva.DMXAdapter) } method from
     * {@code handler} field which can be used to add additional functionality.
     * 
     * @see ServiceTrackerCustomizer#addingService(org.osgi.framework.ServiceReference) 
     * @param sr service reference
     * @return adapter instance
     */
    @Override
    public  DMXAdapter addingService(ServiceReference<DMXAdapter> sr) {
        DMXAdapter adapter = context.getService(sr);
        for(DMXExtensionCallback callback : extensionConfig.getExtensionCallbacks()){
            adapter.registerExtensionCallback(callback);
        }
        
        if(handler != null){
            handler.onAdapterAdded(adapter);
        }
        
        return adapter;
    }

    /**
     * This implementation will invoke {@link DMXAdapterTrackerHandler#onAdapterModified(com.vektorsoft.demux.core.mva.DMXAdapter) } 
     * method, which can be used to add custom functionality.
     * 
     * @param sr service reference
     * @param t adapter object
     */
    @Override
    public  void modifiedService(ServiceReference<DMXAdapter> sr, DMXAdapter t) {
        if(handler != null){
            handler.onAdapterModified(t);
        }
    }

    /**
     * This implementation will invoke {@link DMXAdapterTrackerHandler#onAdapterRemoved(com.vektorsoft.demux.core.mva.DMXAdapter) } 
     * method, which can be used to add custom functionality.
     * 
     * @param sr service reference
     * @param t adapter object
     */
    @Override
    public void removedService(ServiceReference<DMXAdapter> sr, DMXAdapter t) {
        if(handler != null){
            handler.onAdapterRemoved(t);
        }
    }

}
