/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.core.mva;

import com.vektorsoft.demux.core.app.ModelChangeAware;
import com.vektorsoft.demux.core.app.ModelChangeListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 *  Abstract implementation of {@link DMXController}. Clients can extends this class in order to reuse 
 * common functionality found in most controllers, and only implement business logic methods.
 * </p>
 * <p>
 *  In addition, this class implements {@link ModelChangeAware} interface, which enables it to notify listeners about changes in
 * model. This is used to track progress for long-running tasks.
 * </p>
 *
 * @author Vladimir Djurovic
 */
public abstract  class DMXAbstractController implements DMXController, ModelChangeAware {
    
    /** Mapping of model data names and values. */
    protected final Map<String, Object> dataMap;
    
    /** List of model changes listeners. */
    protected List<ModelChangeListener> modelChangeListenersList;
    
    /**
     * Creates new instance.
     */
    public DMXAbstractController(){
        dataMap = new HashMap<String, Object>();
        modelChangeListenersList = new ArrayList<ModelChangeListener>();
    }

    /**
     * This method assumes that no next view is defines, and thus returns an empty string.
     * 
     * @return empty string
     */
    @Override
    public String getNextViewId() {
        return "";
    }

    /**
     * Returns a mapping for this controller. By default, class name is used as mapping.
     * 
     * @return class name as controller mapping
     */
    @Override
    public String getMapping() {
        return getClass().getName();
    }

    /**
     * <p>
     * This method is called before model is updated. Implementations should use this 
     * method as extension point to update {@ #dataMap} with values of fields from this 
     * class which represent values of model data.
     * </p>
     * <p>
     *  Example usage:
     * </p>
     * <code>
     *  private int myInt;
     *  ................
     *  protected void updateDataMapFromFields(){
     *     dataMap.put("myIntMapping", myInt);
     *  }
     * </code>
     */
    protected abstract void updateDataMapFromFields();
    
    
    
    /**
     * <p>
     *  Updates model with values of this class's fields. This method actually returns {@link #dataMap},
     * but it first calls {@link #updateDataMapFromFields() } to update {@code dataMap}.
     * </p>
     * 
     * @return dataMap
     */
    @Override
    public final Map<String, Object> updateModel() {
        updateDataMapFromFields();
        return dataMap;
    }

    /**
     * Returns {@code dataMap} that represents model data mapping.
     * 
     * @return dataMap
     */
    @Override
    public final Map<String, Object> getControllerDataMap() {
        updateDataMapFromFields();
        return dataMap;
    }

    @Override
    public Set<String> getControllerDataNames() {
        return dataMap.keySet();
    }

    @Override
    public void addModelChangeListener(ModelChangeListener listener) {
        modelChangeListenersList.add(listener);
    }

    @Override
    public void removeModelChangeListener(ModelChangeListener listener) {
        modelChangeListenersList.remove(listener);
    }

    @Override
    public void notifyDataChanged(Map<String, Object> data) {
        for(ModelChangeListener listener : modelChangeListenersList){
            listener.modelChanged(data);
        }
    }

}
