/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/

package com.vektorsoft.demux.core.mva;

import com.vektorsoft.demux.core.annotations.DMXControllerDeclaration;
import com.vektorsoft.demux.core.app.ModelChangeAware;
import com.vektorsoft.demux.core.app.ModelChangeListener;
import com.vektorsoft.demux.core.dlg.DialogFactory;
import com.vektorsoft.demux.core.extension.DMXExtendable;
import com.vektorsoft.demux.core.extension.DMXExtensionCallback;
import com.vektorsoft.demux.core.extension.DMXExtensionManager;
import com.vektorsoft.demux.core.extension.DMXExtensionType;
import com.vektorsoft.demux.core.extension.impl.DMXExtensionManagerImpl;
import com.vektorsoft.demux.core.proxy.ControllerInvocationHandler;
import com.vektorsoft.demux.core.proxy.ProxyUtils;
import com.vektorsoft.demux.core.resources.DMXResourceManager;
import com.vektorsoft.demux.core.task.DMXTask;
import com.vektorsoft.demux.core.task.TaskStatusListener;
import java.lang.reflect.Proxy;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.concurrent.ConcurrentSkipListMap;

/**
 * <p> Represents the Adapter in Model-View-Adapter pattern. This class is
 * responsible for communication and synchronization of View and Model. It is
 * also used to register individual controllers and invoke them when
 * appropriate. </p>
 */
public class DMXDefaultAdapter implements Observer, DMXAdapter {

    /**
     * <code>DMXViewManager</code> instance that is used to manage views.
     */
    protected DMXViewManager viewManager;
    /**
     * Map of all registered controllers.
     */
    protected Map<String, DMXController> controllerMap;
    
    /** Model to be used with this adapter. */
    protected DMXModel dataModel;
    
    /** Resource manager used in this adapter. */
    protected DMXResourceManager resourceManager;
    
    /** Extension manager. */
    protected DMXExtensionManager extensionManager;
    
    /**
     * Create new instance.
     * 
     * @param resourceManager resource manager for current platform
     */
    public DMXDefaultAdapter(DMXResourceManager resourceManager) {
        controllerMap = new ConcurrentSkipListMap<String, DMXController>();
        this.dataModel = new DMXModel();
        dataModel.registerObserver(this);
        this.resourceManager = resourceManager;
        this.resourceManager.addLocaleChangeObserver(new ResourceManagerObserver());
        extensionManager = new DMXExtensionManagerImpl();
    }

    /**
     * Sets the current view manager.
     * @param vm <code>DMXViewManager</code> to set
     */
    public void setViewManager(DMXViewManager vm) {
        this.viewManager = vm;
    }

    /**
     * Sets current dataModel.
     * 
     * @param model dataModel to set
     */
    public void setModel(DMXModel model) {
        this.dataModel = model;
        dataModel.registerObserver(this);
    }
    
    
    @Override
    public DMXResourceManager getResourceManager() {
        return resourceManager;
    }

    
    @Override
    public DMXExtensionManager getExtensionManager() {
        return extensionManager;
    }
    

    
    @Override
    public void registerController(DMXController controller) {
        //check parameters validity
        if (controller == null) {
            throw new IllegalArgumentException("Controller cannot be null.");
        }
        if (controller.getMapping() == null || "".equals(controller.getMapping())) {
            throw new IllegalArgumentException("Controller mapping cannot be null or empty string.");
        }

        if(controller instanceof DMXExtendable){
            extensionManager.registerExtendableController((DMXExtendable)controller);
        }
        
        controllerMap.put(controller.getMapping(), controller);
        for (Map.Entry<String, Object> entry : getControllerData(controller).entrySet()) {
            dataModel.registerData(entry.getKey(), entry.getValue());
        }

    }
    
    
    @Override
    public void registerController(Class clazz){
        //check parameters validity
        if(clazz == null){
            throw new IllegalArgumentException("Controller cannot be null.");
        }
        // check annotations
        if(clazz.isAnnotationPresent(DMXControllerDeclaration.class)){
            try {
                
                ControllerInvocationHandler handler = new ControllerInvocationHandler(clazz);
                List<Class> interfaces = new ArrayList<Class>();
                interfaces.add(DMXController.class);
                interfaces.addAll(ProxyUtils.getAllImplementedInterfaces(clazz));
                Object obj = Proxy.newProxyInstance(this.getClass().getClassLoader(), interfaces.toArray(new Class[interfaces.size()]), handler);
                registerController((DMXController)obj);
                
            } catch(Exception ex){
                throw new RuntimeException(ex);
            }
            
        }
    }
    
    @Override
    public void registerView(DMXView view) {
        viewManager.registerView(view);
        afterViewRegistration(view);
    }
    
    @Override
    public void registerView(Class clazz){
        DMXView view = viewManager.registerView(clazz);
        if(view != null){
            afterViewRegistration(view);
        }
        
    }
    
    /**
     * Called after successful view registration to update model and
     * extensions cache.
     * 
     * @param view registered view
     */
    private void afterViewRegistration(DMXView view){
        if(view instanceof DMXExtendable){
            extensionManager.registerExtendableView((DMXExtendable)view, viewManager);
        }
        Set<String> ids = view.getViewDataIds();
        if(ids != null && !ids.isEmpty()){
            viewManager.updateViewData(dataModel.getDataValues(view.getViewDataIds().toArray(new String[1])));
        }
    }

    @Override
    public void registerExtensionCallback(DMXExtensionCallback callback) {
        if(extensionManager == null){
            throw new IllegalStateException("Extension manager is not set");
        }
        String[] mappings = callback.getMappings();
        DMXExtensionType type = callback.getExtensionType();
        
        // store callback in extension manager cache
        extensionManager.registerCallback(callback);
        resourceManager.registerCallbackResources(callback);
        
        for(String mapping : mappings){
            if(type == DMXExtensionType.CONTROLLER_EXTENSION){
                DMXController ctrl = controllerMap.get(mapping);
                if(ctrl != null && ctrl instanceof DMXExtendable){
                    ((DMXExtendable)ctrl).setExtensionCallback(callback);
                    for (Map.Entry<String, Object> entry : callback.getCallbackData().entrySet()) {
                        dataModel.registerData(entry.getKey(), entry.getValue());
                    }
                } 
            }else if(type == DMXExtensionType.VIEW_EXTENSION){
                viewManager.setViewExtensionCallback(mapping, callback);
            }
        }
        
    }
    
    @Override
    public <T extends DialogFactory> T getDialogFactory(Class<? extends DialogFactory> clazz){
        return viewManager.getDialogFactory();
    }
    
    /**
     * Extracts data from controller that should be register with the dataModel.
     *
     * @param controller controller
     * @return map of controller data
     */
    protected Map<String, Object> getControllerData(DMXController controller) {
        Map<String, Object> map = controller.getControllerDataMap();
        // add additonal data from callback, if any
        if(controller instanceof DMXExtendable){
            DMXExtensionCallback callback = ((DMXExtendable)controller).getExtensionCallback();
            if(callback != null){
                Map<String, Object> dm = callback.getCallbackData();
                if(dm != null && !dm.isEmpty()){
                    map.putAll(dm);
                }
            }
        }
       return controller.getControllerDataMap();
    }

    /**
     * <p>
     * Invoked when any of the dataModel data changes. Adapter will notify view manager, which
     *  will in turn update views that depend on changed data.
     * </p>
     * <p>
     *  Argument must be an instance of {@code java.util.Map}, or the {@code IllegalArgumentException} 
     * will be thrown.
     * </p>
     * 
     * @param o source of the change
     * @param arg optional argument
     */
    @Override
    public void update(Observable o, Object arg) {
        // assume that argument is map
        if(arg instanceof Map){
            viewManager.updateViewData((Map<String, Object>)arg);
        } else {
            throw new IllegalArgumentException("Argument must be an instance of java.util.Map");
        }
    }
    
    /**
     * Prepares controller for execution. This method will find controller by it's ID and inject any 
     * model data required.
     * 
     * @param controllerId controller ID
     * @return controller instance
     * @throws NoSuchControllerException if no controller with specified ID is found
     */
    private DMXController prepareController(String controllerId) throws NoSuchControllerException {
        DMXController controller = controllerMap.get(controllerId);
        if (controller == null) {
            throw new NoSuchControllerException("Controller not found: " + controllerId);
        }
        // need to use separate Set here, otherwise UnsupportedOperationException is thrown
        // in AbstractCollection.addAll() method
        Set<String> names = new HashSet<String>();
        names.addAll(controller.getControllerDataNames());
        
        // get callback data, if any
        if(controller instanceof DMXExtendable){
            DMXExtensionCallback callback = ((DMXExtendable)controller).getExtensionCallback();
            if(callback != null && callback.getCallbackData() != null){
                names.addAll(callback.getCallbackData().keySet());
            }
        }
        Map<String, Object> dataMap = dataModel.getDataValues(names.toArray(new String[names.size()]));

        controller.updateFromModel(dataMap);
        // update data in callback
        if(controller instanceof DMXExtendable){
            DMXExtensionCallback callback = ((DMXExtendable)controller).getExtensionCallback();
            if(callback != null){
                callback.setDataValuesFromModel(dataMap);
            }
        }
        return controller;
    }
    
     
    @Override
    public void invokeController(String controllerId, Object... args) {
        try {
            DMXController controller = prepareController(controllerId);

            controller.execute(args);
            dataModel.setDataValues(controller.updateModel());
            // update model from callback data, if any
            if (controller instanceof DMXExtendable) {
                DMXExtensionCallback callback = ((DMXExtendable) controller).getExtensionCallback();
                if (callback != null && callback.getCallbackData() != null) {
                    dataModel.setDataValues(callback.getCallbackData());
                }
            }
            // set active view
            if(args.length > 1 && args[1] instanceof Map){
                viewManager.selectView(controller.getNextViewId(), (Map<String, Object>)args[1]);
            } else {
                viewManager.selectView(controller.getNextViewId(), null);
            }
            

        } catch (ExecutionException ex) {
            throw new RuntimeException(ex);
        } catch (NoSuchControllerException nex) {
            throw new RuntimeException(nex);
        }
    }
    
    
    @Override
    public void executeTask(DMXTask task, Object... args) {
        try {
            DMXController controller = prepareController(task.getControllerId());
           
            TaskListener taskListener = new TaskListener();
            // find progress view, if any
            String viewId = task.getProgressViewId();
            taskListener.setViewId(viewId);
            if(task.isProgressAware() && controller instanceof ModelChangeAware){
                ((ModelChangeAware)controller).addModelChangeListener(taskListener);
            }
            task.addTaskStatusListener(taskListener);
            task.setController(controller);
            task.setArguments(args);
            
            
            Thread th = new Thread(task);
            th.start();
            
        } catch (NoSuchControllerException ex) {
            throw new RuntimeException(ex);
        } 
    }

    @Override
    public void registerEventHandler(DMXEventRegistrationHandler handler) {
        viewManager.registerEventHandler(handler);
    }
    
    
    
    /**
     * Observer for {@code DMXResourceManager}. It will receive notifications of locale changes.
     */
    private class ResourceManagerObserver implements Observer {

        @Override
        public void update(Observable o, Object arg) {
            viewManager.handleLocaleChange();
        }
    }
    
    /**
     * Implementation of {@code ModelChangeListener}. This implementation will notify view of any 
     * changes of model data, so it can be immediately displayed.
     */
    private class TaskListener implements ModelChangeListener, TaskStatusListener {
        
        /** ID of the view used to display progress. */
        private String viewId;
        
        /** Parameters map. */
        private Map<String, Object> params = new HashMap<String, Object>();

        /**
         * Set view ID.
         * 
         * @param viewId  view ID
         */
        public void setViewId(String viewId) {
            this.viewId = viewId;
        }
        

        @Override
        public void modelChanged(Map<String, Object> modelData) {
            dataModel.setDataValues(modelData);
        }

        @Override
        public void taskStarted(DMXTask task) {
            viewManager.blockGui(true, task.getBlockingScope(), task.getIdsToBlock());
            if(viewId != null){
                params.put(DMXViewManager.PARAM_PROGRESS_DIALOG, DMXViewManager.VAL_DLG_SHOW);
                viewManager.selectView(viewId, params);
            }
            
        }

        @Override
        public void taskCompleted(DMXTask task, Object result) {
            viewManager.blockGui(false, task.getBlockingScope(), task.getIdsToBlock());
             if(viewId != null){
                params.put(DMXViewManager.PARAM_PROGRESS_DIALOG, DMXViewManager.VAL_DLG_CLOSE);
                viewManager.selectView(viewId, params);
            }
        }
        
    }
}
